@@ -1,47 +1,53 @@
-# encoding: UTF-8
+# frozen_string_literal: true
 module Asciidoctor
 class AbstractBlock < AbstractNode
-  # Public: Get the Array of Asciidoctor::AbstractBlock sub-blocks for this block
+  # Public: Get the Array of {AbstractBlock} child blocks for this block. Only applies if content model is :compound.
   attr_reader :blocks
 
-  # Public: Set the caption for this block
+  # Public: Set the caption for this block.
   attr_writer :caption
 
-  # Public: The types of content that this block can accomodate
+  # Public: Describes the type of content this block accepts and how it should be converted. Acceptable values are:
+  # * :compound - this block contains other blocks
+  # * :simple - this block holds a paragraph of prose that receives normal substitutions
+  # * :verbatim - this block holds verbatim text (displayed "as is") that receives verbatim substitutions
+  # * :raw - this block holds unprocessed content passed directly to the output with no sustitutions applied
+  # * :empty - this block has no content
   attr_accessor :content_model
 
-  # Public: Set the Integer level of this Section or the Section level in which this Block resides
+  # Public: Set the Integer level of this {Section} or the level of the Section to which this {AbstractBlock} belongs.
   attr_accessor :level
 
-  # Public: Get/Set the number of this block (if section, relative to parent, otherwise absolute)
-  # Only assigned to section if automatic section numbering is enabled
-  # Only assigned to formal block (block with title) if corresponding caption attribute is present
-  attr_accessor :number
+  # Public: Get/Set the String numeral of this block (if section, relative to parent, otherwise absolute).
+  # Only assigned to section if automatic section numbering is enabled.
+  # Only assigned to formal block (block with title) if corresponding caption attribute is present.
+  attr_accessor :numeral
 
-  # Public: Gets/Sets the location in the AsciiDoc source where this block begins
+  # Public: Gets/Sets the location in the AsciiDoc source where this block begins.
   attr_accessor :source_location
 
   # Public: Get/Set the String style (block type qualifier) for this block.
   attr_accessor :style
 
-  # Public: Substitutions to be applied to content in this block
+  # Public: Substitutions to be applied to content in this block.
   attr_reader :subs
 
+  # Internal: Set the default subs applied to this block; used during block creation
+  attr_writer :default_subs
+
   def initialize parent, context, opts = {}
     super
     @content_model = :compound
     @blocks = []
     @subs = []
-    @id = @title = @title_converted = @caption = @number = @style = @default_subs = @source_location = nil
-    if context == :document
-      @level = 0
-    elsif parent && context != :section
-      @level = parent.level
+    @id = @title = @caption = @numeral = @style = @default_subs = @source_location = nil
+    case context
+    when :document, :section
+      @level = @next_section_index = 0
+      @next_section_ordinal = 1
     else
-      @level = nil
+      @level = AbstractBlock === parent ? parent.level : nil
     end
-    @next_section_index = 0
-    @next_section_number = 1
   end
 
   def block?
@@ -77,7 +83,7 @@ class AbstractBlock < AbstractNode
   # Public: Get the converted result of the child blocks by converting the
   # children appropriate to content model that this block supports.
   def content
-    @blocks.map {|b| b.convert } * LF
+    @blocks.map {|b| b.convert }.join LF
   end
 
   # Public: Update the context of this block.
@@ -97,10 +103,10 @@ class AbstractBlock < AbstractNode
   #
   # Examples
   #
-  #   block = Block.new(parent, :preamble, :content_model => :compound)
+  #   block = Block.new(parent, :preamble, content_model: :compound)
   #
-  #   block << Block.new(block, :paragraph, :source => 'p1')
-  #   block << Block.new(block, :paragraph, :source => 'p2')
+  #   block << Block.new(block, :paragraph, source: 'p1')
+  #   block << Block.new(block, :paragraph, source: 'p2')
   #   block.blocks?
   #   # => true
   #   block.blocks.size
@@ -132,10 +138,19 @@ class AbstractBlock < AbstractNode
     @next_section_index > 0
   end
 
-  # Public: Query for all descendant block-level nodes in the document tree
-  # that match the specified selector (context, style, id, and/or role). If a
-  # Ruby block is given, it's used as an additional filter. If no selector or
-  # Ruby block is supplied, all block-level nodes in the tree are returned.
+  # Deprecated: Legacy property to get the String or Integer numeral of this section.
+  def number
+    (Integer @numeral) rescue @numeral
+  end
+
+  # Public: Walk the document tree and find all block-level nodes that match the specified selector (context, style, id,
+  # role, and/or custom filter).
+  #
+  # If a Ruby block is given, it's applied as a supplemental filter. If the filter returns true (which implies :accept),
+  # the node is accepted and node traversal continues. If the filter returns false (which implies :skip), the node is
+  # skipped, but its children are still visited. If the filter returns :reject, the node and all its descendants are
+  # rejected. If the filter returns :prune, the node is accepted, but its descendants are rejected. If no selector
+  # or filter block is supplied, all block-level nodes in the tree are returned.
   #
   # Examples
   #
@@ -153,46 +168,8 @@ class AbstractBlock < AbstractNode
   #--
   # TODO support jQuery-style selector (e.g., image.thumb)
   def find_by selector = {}, &block
-    result = []
-
-    if ((any_context = !(context_selector = selector[:context])) || context_selector == @context) &&
-        (!(style_selector = selector[:style]) || style_selector == @style) &&
-        (!(role_selector = selector[:role]) || (has_role? role_selector)) &&
-        (!(id_selector = selector[:id]) || id_selector == @id)
-      if id_selector
-        if block_given?
-          return (yield self) ? [self] : result
-        else
-          return [self]
-        end
-      elsif block_given?
-        result << self if (yield self)
-      else
-        result << self
-      end
-    end
-
-    # process document header as a section if present
-    if @context == :document && (any_context || context_selector == :section) && header?
-      result.concat(@header.find_by selector, &block)
-    end
-
-    unless context_selector == :document # optimization
-      # yuck, dlist is a special case
-      if @context == :dlist
-        if any_context || context_selector != :section # optimization
-          @blocks.flatten.each do |li|
-            # NOTE the list item of a dlist can be nil, so we have to check
-            result.concat(li.find_by selector, &block) if li
-          end
-        end
-      elsif
-        @blocks.each do |b|
-          next if (context_selector == :section && b.context != :section) # optimization
-          result.concat(b.find_by selector, &block)
-        end
-      end
-    end
+    find_by_internal selector, (result = []), &block
+  rescue ::StopIteration
     result
   end
 
@@ -201,7 +178,13 @@ class AbstractBlock < AbstractNode
   # Move to the next adjacent block in document order. If the current block is the last
   # item in a list, this method will return the following sibling of the list block.
   def next_adjacent_block
-    (sib = (p = parent).blocks[(p.blocks.find_index self) + 1]) ? sib : p.next_adjacent_block unless @context == :document
+    unless @context == :document
+      if (p = @parent).context == :dlist && @context == :list_item
+        (sib = p.items[(p.items.find_index {|terms, desc| (terms.include? self) || desc == self }) + 1]) ? sib : p.next_adjacent_block
+      else
+        (sib = p.blocks[(p.blocks.find_index self) + 1]) ? sib : p.next_adjacent_block
+      end
+    end
   end
 
   # Public: Get the Array of child Section objects
@@ -212,12 +195,12 @@ class AbstractBlock < AbstractNode
   #
   #   doc << (sect1 = Section.new doc, 1)
   #   sect1.title = 'Section 1'
-  #   para1 = Block.new sect1, :paragraph, :source => 'Paragraph 1'
-  #   para2 = Block.new sect1, :paragraph, :source => 'Paragraph 2'
+  #   para1 = Block.new sect1, :paragraph, source: 'Paragraph 1'
+  #   para2 = Block.new sect1, :paragraph, source: 'Paragraph 2'
   #   sect1 << para1 << para2
   #   sect1 << (sect1_1 = Section.new sect1, 2)
   #   sect1_1.title = 'Section 1.1'
-  #   sect1_1 << (Block.new sect1_1, :paragraph, :source => 'Paragraph 3')
+  #   sect1_1 << (Block.new sect1_1, :paragraph, source: 'Paragraph 3')
   #   sect1.blocks?
   #   # => true
   #   sect1.blocks.size
@@ -296,7 +279,7 @@ class AbstractBlock < AbstractNode
   # Returns the converted String title for this Block, or nil if the source title is falsy
   def title
     # prevent substitutions from being applied to title multiple times
-    @title_converted ? @converted_title : (@converted_title = (@title_converted = true) && @title && (apply_title_subs @title))
+    @converted_title ||= @title && (apply_title_subs @title)
   end
 
   # Public: A convenience method that checks whether the title of this block is defined.
@@ -310,7 +293,8 @@ class AbstractBlock < AbstractNode
   #
   # Returns the new String title assigned to this Block
   def title= val
-    @title, @title_converted = val, nil
+    @converted_title = nil
+    @title = val
   end
 
   # Public: A convenience method that checks whether the specified
@@ -357,15 +341,15 @@ class AbstractBlock < AbstractNode
     elsif xrefstyle && @title && @caption
       case xrefstyle
       when 'full'
-        quoted_title = sprintf sub_quotes(@document.compat_mode ? %q(``%s'') : '"`%s`"'), title
-        if @number && (prefix = @document.attributes[@context == :image ? 'figure-caption' : %(#{@context}-caption)])
-          %(#{prefix} #{@number}, #{quoted_title})
+        quoted_title = sub_placeholder (sub_quotes @document.compat_mode ? %q(``%s'') : '"`%s`"'), title
+        if @numeral && (prefix = @document.attributes[@context == :image ? 'figure-caption' : %(#{@context}-caption)])
+          %(#{prefix} #{@numeral}, #{quoted_title})
         else
           %(#{@caption.chomp '. '}, #{quoted_title})
         end
       when 'short'
-        if @number && (prefix = @document.attributes[@context == :image ? 'figure-caption' : %(#{@context}-caption)])
-          %(#{prefix} #{@number})
+        if @numeral && (prefix = @document.attributes[@context == :image ? 'figure-caption' : %(#{@context}-caption)])
+          %(#{prefix} #{@numeral})
         else
           @caption.chomp '. '
         end
@@ -391,13 +375,13 @@ class AbstractBlock < AbstractNode
   # value - The explicit String caption to assign to this block (default: nil).
   # key   - The String prefix for the caption and counter attribute names.
   #         If not provided, the name of the context for this block is used.
-  #         (default: nil)
+  #         (default: @context)
   #
   # Returns nothing.
-  def assign_caption value = nil, key = nil
+  def assign_caption value = nil, key = @context
     unless @caption || !@title || (@caption = value || @document.attributes['caption'])
-      if (prefix = @document.attributes[%(#{key ||= @context}-caption)])
-        @caption = %(#{prefix} #{@number = @document.increment_and_store_counter "#{key}-number", self}. )
+      if (prefix = @document.attributes[%(#{key}-caption)])
+        @caption = %(#{prefix} #{@numeral = @document.increment_and_store_counter %[#{key}-number], self}. )
         nil
       end
     end
@@ -417,17 +401,14 @@ class AbstractBlock < AbstractNode
     @next_section_index = (section.index = @next_section_index) + 1
     if (like = section.numbered)
       if (sectname = section.sectname) == 'appendix'
-        section.number = @document.counter 'appendix-number', 'A'
-        if (caption = @document.attributes['appendix-caption'])
-          section.caption = %(#{caption} #{section.number}: )
-        else
-          section.caption = %(#{section.number}. )
-        end
+        section.numeral = @document.counter 'appendix-number', 'A'
+        section.caption = (caption = @document.attributes['appendix-caption']) ? %(#{caption} #{section.numeral}: ) : %(#{section.numeral}. )
       # NOTE currently chapters in a book doctype are sequential even for multi-part books (see #979)
       elsif sectname == 'chapter' || like == :chapter
-        section.number = @document.counter 'chapter-number', 1
+        section.numeral = (@document.counter 'chapter-number', 1).to_s
       else
-        @next_section_number = (section.number = @next_section_number) + 1
+        section.numeral = sectname == 'part' ? (Helpers.int_to_roman @next_section_ordinal) : @next_section_ordinal.to_s
+        @next_section_ordinal += 1
       end
     end
     nil
@@ -445,7 +426,7 @@ class AbstractBlock < AbstractNode
   # Returns nothing
   def reindex_sections
     @next_section_index = 0
-    @next_section_number = 1
+    @next_section_ordinal = 1
     @blocks.each do |block|
       if block.context == :section
         assign_numeral block
@@ -454,35 +435,74 @@ class AbstractBlock < AbstractNode
     end
   end
 
-# stage the Enumerable mixin until we're sure we've got it right
-=begin
-  include ::Enumerable
-
-  # Public: Yield the block on this block node and all its descendant
-  # block node children to satisfy the Enumerable contract.
-  #
-  # Returns nothing
-  def each &block
-    # yucky, dlist is a special case
-    if @context == :dlist
-      @blocks.flatten.each &block
+  # Internal: Performs the work for find_by, but does not handle the StopIteration exception.
+  protected def find_by_internal selector = {}, result = [], &block
+    if ((any_context = (context_selector = selector[:context]) ? nil : true) || context_selector == @context) &&
+        (!(style_selector = selector[:style]) || style_selector == @style) &&
+        (!(role_selector = selector[:role]) || (has_role? role_selector)) &&
+        (!(id_selector = selector[:id]) || id_selector == @id)
+      if block_given?
+        if (verdict = yield self)
+          case verdict
+          when :prune
+            result << self
+            raise ::StopIteration if id_selector
+            return result
+          when :reject
+            raise ::StopIteration if id_selector
+            return result
+          when :stop
+            raise ::StopIteration
+          else
+            result << self
+            raise ::StopIteration if id_selector
+          end
+        elsif id_selector
+          raise ::StopIteration
+        end
+      else
+        result << self
+        raise ::StopIteration if id_selector
+      end
+    end
+    case @context
+    when :document
+      unless context_selector == :document
+        # process document header as a section, if present
+        if header? && (any_context || context_selector == :section)
+          @header.find_by_internal selector, result, &block
+        end
+        @blocks.each do |b|
+          next if (context_selector == :section && b.context != :section) # optimization
+          b.find_by_internal selector, result, &block
+        end
+      end
+    when :dlist
+      # dlist has different structure than other blocks
+      if any_context || context_selector != :section # optimization
+        # NOTE the list item of a dlist can be nil, so we have to check
+        @blocks.flatten.each {|b| b.find_by_internal selector, result, &block if b }
+      end
+    when :table
+      if selector[:traverse_documents]
+        rows.head.each {|r| r.each {|c| c.find_by_internal selector, result, &block } }
+        selector = selector.merge context: :document if context_selector == :inner_document
+        (rows.body + rows.foot).each do |r|
+          r.each do |c|
+            c.find_by_internal selector, result, &block
+            c.inner_document.find_by_internal selector, result, &block if c.style == :asciidoc
+          end
+        end
+      else
+        (rows.head + rows.body + rows.foot).each {|r| r.each {|c| c.find_by_internal selector, result, &block } }
+      end
     else
-      #yield self.header if @context == :document && header?
-      @blocks.each &block
+      @blocks.each do |b|
+        next if (context_selector == :section && b.context != :section) # optimization
+        b.find_by_internal selector, result, &block
+      end
     end
+    result
   end
-
-  #--
-  # TODO is there a way to make this lazy?
-  def each_recursive &block
-    block = lambda {|node| node } unless block_given?
-    results = []
-    self.each do |node|
-      results << block.call(node)
-      results.concat(node.each_recursive(&block)) if ::Enumerable === node
-    end
-    block_given? ? results : results.to_enum
-  end
-=end
 end
 end
